from typing import Optional, TYPE_CHECKING

from telefone.bot.blueprint.abc import ABCBlueprint
from telefone.bot.dispatch.labelers.default import Labeler
from telefone.bot.dispatch.router.base import Router
from telefone.modules import logger

if TYPE_CHECKING:
    from telefone.api import ABCAPI
    from telefone.bot.bot import Bot
    from telefone.bot.polling.abc import ABCPolling


class BotBlueprint(ABCBlueprint):
    def __init__(
        self,
        name: Optional[str] = None,
        labeler: Optional[Labeler] = None,
        router: Optional[Router] = None,
    ):
        if name is not None:
            self.name = name

        self.labeler = labeler or Labeler()
        self.router: Router = router or Router()
        self.constructed = False

    def construct(self, api: "ABCAPI", polling: "ABCPolling") -> "BotBlueprint":
        self.api = api
        self.polling = polling
        self.constructed = True
        return self

    def load(self, framework: "Bot") -> "BotBlueprint":
        framework.labeler.load(self.labeler)  # type: ignore
        logger.debug(f"Blueprint {self.name!r} loaded")
        return self.construct(framework.api, framework.polling)

    @property
    def on(self) -> Labeler:
        return self.labeler
